package com.example.shawarma.screens.cart

import androidx.compose.foundation.BorderStroke
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.verticalScroll
import androidx.compose.material.Button
import androidx.compose.material.ButtonDefaults
import androidx.compose.material.Card
import androidx.compose.material.Icon
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.MutableState
import androidx.compose.runtime.mutableStateListOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.compose.ui.zIndex
import com.example.shawarma.R
import com.example.shawarma.data.db.AppDatabase
import com.example.shawarma.data.models.OrderStatus
import com.example.shawarma.data.models.OrderWithProducts
import com.example.shawarma.ui.theme.MarckFamily
import com.example.shawarma.ui.theme.MyLightRed
import com.example.shawarma.ui.theme.MyMainBackground
import com.example.shawarma.ui.theme.MyOrange
import com.example.shawarma.ui.theme.NunitoFamily
import com.example.shawarma.widgets.ShawarmaLogo2
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext

@Composable
fun CartScreen() {
    Box(
        contentAlignment = Alignment.TopCenter
    ) {
        CartWidget()
        ShawarmaLogo2()
    }
}

@Composable
fun CartWidget(){

    val context = LocalContext.current
    val orders = remember { mutableStateListOf<OrderWithProducts>() }
    LaunchedEffect(Unit) {
        withContext(Dispatchers.IO) {
            AppDatabase.getInstance(context).orderDao().getByUserId(1).collect { data ->
                orders.clear()
                orders.addAll(data)
            }
        }
    }

    var unpaidOrder: OrderWithProducts? = null
    val preparingOrders = mutableListOf<OrderWithProducts>()

    for(order in orders) {
        if (order.order.status == OrderStatus.Неоплачено.toString()) {
            unpaidOrder = order
        }
        if (order.order.status == OrderStatus.Готовится.toString() || order.order.status == OrderStatus.Готово.toString()) {
            preparingOrders.add(order)
        }
    }

    Box(
        modifier = Modifier
            .clip(shape = RoundedCornerShape(30.dp))
            .padding(top = 100.dp)
            .fillMaxSize()
            .background(color = MyMainBackground)
            .zIndex(2f),

        contentAlignment = Alignment.TopCenter
    ) {
        Text(
            text = "Корзина",
            fontFamily = MarckFamily,
            fontSize = 40.sp,
            modifier = Modifier.padding(top = 15.dp)
        )
        Column(
            horizontalAlignment = Alignment.CenterHorizontally,
            modifier = Modifier
                .fillMaxWidth()
                .padding(top = 55.dp)
                .verticalScroll(rememberScrollState())
        ) {

            Text(
                text = "Оплачено:",
                fontFamily = NunitoFamily,
                fontSize = 20.sp,
                modifier = Modifier.padding(top = 15.dp),
                fontWeight = FontWeight.Bold
            )
            LazyColumn(
                modifier = Modifier
                    .width(340.dp)
                    .height(200.dp)
            ) {
                items(preparingOrders.size) { index ->
                        PaidItem(preparingOrders[index])
                }
            }
            Text(
                text = "Ожидает оплаты:",
                fontFamily = NunitoFamily,
                fontSize = 20.sp,
                modifier = Modifier.padding(top = 15.dp),
                fontWeight = FontWeight.Bold
            )
            if (unpaidOrder != null) {
                CartItem(order = unpaidOrder)
            }

            Spacer(modifier = Modifier.height(20.dp))

            Button(
                colors = ButtonDefaults.buttonColors(
                    backgroundColor = Color(0xFF91FF87)
                ),
                shape = RoundedCornerShape(20.dp),
                border = BorderStroke(2.dp, Color(0x66000000)),
                modifier = Modifier.size(240.dp, 60.dp),
                onClick = { /*TODO*/ }
            ) {
                Text(
                    "Оплатить",
                    fontSize = 20.sp,
                    fontFamily = NunitoFamily,
                    fontWeight = FontWeight(700),
                )
            }

            Spacer(modifier = Modifier.height(70.dp))


        }
    }
}

@Composable
fun PaidItem(order : OrderWithProducts) {
    Card(
        border = BorderStroke(width = 2.dp, color = MyOrange),
        shape = RoundedCornerShape(size = 20.dp),
        backgroundColor = Color.White,
        modifier = Modifier.size(340.dp, 100.dp)
    )  {
        LazyColumn {
            items(order.orderWithProducts.size) { index ->
                if (index == 0) {
                    Row(
                        horizontalArrangement = Arrangement.SpaceBetween,
                        modifier = Modifier
                            .fillMaxWidth()
                            .padding(start = 20.dp, end = 20.dp)
                    ){
                        Text(
                            text = "Статус: ",
                            fontFamily = NunitoFamily,
                            fontSize = 20.sp,
                            modifier = Modifier.padding(top = 15.dp),
                            fontWeight = FontWeight.Bold,
                            color = Color.Gray
                        )
                        Text(
                            text = order.order.status,
                            fontFamily = NunitoFamily,
                            fontSize = 20.sp,
                            modifier = Modifier.padding(top = 15.dp),
                            fontWeight = FontWeight.Bold,
                            color = Color.Gray
                        )
                    }
                }
                Row(
                    horizontalArrangement = Arrangement.SpaceBetween,
                    modifier = Modifier
                        .fillMaxWidth()
                        .padding(horizontal = 20.dp)
                ) {
                    Text(
                        text = order.orderWithProducts[index].product.title,
                        fontFamily = NunitoFamily,
                        fontSize = 20.sp,
                        modifier = Modifier.padding(top = 15.dp),
                        fontWeight = FontWeight.Bold
                    )
                    Text(
                        text = "x" + order.orderWithProducts[index].orderProductModel.quantity,
                        fontFamily = NunitoFamily,
                        fontSize = 20.sp,
                        modifier = Modifier.padding(top = 15.dp),
                        fontWeight = FontWeight.Bold
                    )
                    Text(
                        text = order.orderWithProducts[index].orderProductModel.totalPrice.toString() + " руб.",
                        fontFamily = NunitoFamily,
                        fontSize = 20.sp,
                        modifier = Modifier.padding(top = 15.dp),
                        fontWeight = FontWeight.Bold
                    )
                }
            }
        }
    }
    Spacer(modifier = Modifier.height(20.dp))
}

@Composable
fun CartItem(order : OrderWithProducts) {
    Card(
        border = BorderStroke(width = 2.dp, color = MyOrange),
        shape = RoundedCornerShape(size = 20.dp),
        backgroundColor = Color.White,
        modifier = Modifier.size(340.dp, 180.dp)
    ) {
        LazyColumn (
            horizontalAlignment = Alignment.CenterHorizontally
        ) {
            items(order.orderWithProducts.size) {index ->
                Row(
                    verticalAlignment = Alignment.CenterVertically,
                    horizontalArrangement = Arrangement.SpaceBetween,
                    modifier = Modifier.padding(20.dp).size(340.dp, 60.dp)
                ) {
                    Column(
                        modifier = Modifier.fillMaxWidth(0.5f)
                    ) {
                        Row(
                            horizontalArrangement = Arrangement.SpaceBetween,
                            modifier = Modifier.fillMaxWidth()
                        ) {
                            Text(
                                text = order.orderWithProducts[index].product.title,
                                fontFamily = NunitoFamily,
                                fontSize = 20.sp,
                                fontWeight = FontWeight.Bold
                            )
                            Text(
                                text = "x" + order.orderWithProducts[index].orderProductModel.quantity,
                                fontFamily = NunitoFamily,
                                fontSize = 20.sp,
                                fontWeight = FontWeight.Bold
                            )
                        }
                        Text(
                            text = order.orderWithProducts[index].orderProductModel.totalPrice.toString() + " руб.",
                            fontFamily = NunitoFamily,
                            fontSize = 20.sp,
                            fontWeight = FontWeight.Bold
                        )
                    }
                    Button(
                        colors = ButtonDefaults.buttonColors(
                            backgroundColor = MyLightRed
                        ),
                        shape = RoundedCornerShape(size = 10.dp),
                        modifier = Modifier
                            .size(100.dp, 60.dp)
                            .fillMaxSize(0.5f),
                        onClick = { /*TODO*/ }
                    ) {
                        Icon(
                            painter = painterResource(id = R.drawable.trash),
                            contentDescription = "Delete",
                            modifier = Modifier.size(42.dp)
                        )
                    }

                }
            }

        }
    }
}